/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import de.ozgcloud.command.CommandExecutedEvent;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.command.CommandRevokeFailedEvent;
import de.ozgcloud.command.CommandRevokedEvent;
import lombok.extern.log4j.Log4j2;

@Component
@Log4j2
public class CommandEventListener {

	@Autowired
	private CommandService commandService;

	@EventListener
	public void setStatusFinished(CommandExecutedEvent event) {
		commandService.setCommandFinished(event.getSource(), event.getCreatedResource());
	}

	@EventListener
	public void setStatusFinished(CommandRevokeFailedEvent event) {
		commandService.setCommandFinished(event.getSource());
	}

	@EventListener
	public void setStatusError(CommandFailedEvent event) {
		commandService.setCommandError(event.getSource(), event.getErrorMessage());
	}

	@EventListener
	public void setCommandRevoked(CommandRevokedEvent event) {
		commandService.setCommandRevoked(event.getSource().getId());
	}

}
