/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.db;

import java.util.ConcurrentModificationException;

import com.mongodb.Function;
import com.mongodb.client.result.DeleteResult;
import com.mongodb.client.result.UpdateResult;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class CollisionVerifier {

	private final Function<String, Boolean> doEntryWithIdExist;

	public void verify(DeleteResult result, String entryId) {
		if (isConcurrentModification(result) && doEntryWithIdExist(entryId)) {
			throw new ConcurrentModificationException();
		}
	}

	public void verify(UpdateResult result, String entryId) {
		if (isConcurrentModification(result) && doEntryWithIdExist(entryId)) {
			throw new ConcurrentModificationException();
		}
	}

	boolean doEntryWithIdExist(String entryId) {
		return doEntryWithIdExist.apply(entryId);
	}

	boolean isConcurrentModification(DeleteResult result) {
		return result.wasAcknowledged() && result.getDeletedCount() <= 0;
	}

	boolean isConcurrentModification(UpdateResult result) {
		return result.getModifiedCount() <= 0 && result.getMatchedCount() <= 0;
	}
}
