/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.errorhandling;

import java.util.Map;

import de.ozgcloud.common.errorhandling.FunctionalErrorCode;
import lombok.Getter;

@Getter
public class NotFoundException extends FunctionalException {

	static final String KEY_ENTITY_NAME = "EntityName";
	static final String KEY_ID = "Id";

	private static final long serialVersionUID = 1L;

	private static final String MESSAGE_TEMPLATE = "Object '%s' with id '%s' not found.";
	private static final FunctionalErrorCode ERROR_CODE = () -> "error.not_found";

	public NotFoundException(Class<?> resource, Object id) {
		super(String.format(MESSAGE_TEMPLATE, resource.getSimpleName(), id.toString()), ERROR_CODE,
				Map.of(KEY_ENTITY_NAME, resource.getSimpleName(), KEY_ID, id.toString()));
	}

}