/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;

@ChangeUnit(id = "2024-04-02 17:15:00 OZG-4770", order = "M009", author = "ebardin", runAlways = true)
public class M009_SetServiceKontoTrustLevel { // NOSONAR

	private static final String VORGANG_COLLECTION = "vorgang";
	private static final String DEFAULT_TRUST_LEVEL = "STORK-QAA-Level-1";
	private static final String KEY_SERVICE_KONTO = "header.serviceKonto";
	private static final String KEY_TRUST_LEVEL = KEY_SERVICE_KONTO + ".trustLevel";

	@Execution
	public void doMigration(MongoOperations db) {
		db.updateMulti(createFilterQuery(), createUpdate(), VORGANG_COLLECTION);
	}

	Query createFilterQuery() {
		var serviceKontoExistsCriteria = Criteria.where(KEY_SERVICE_KONTO).exists(true);
		var trustLevelNotExistsCriteria = Criteria.where(KEY_TRUST_LEVEL).exists(false);
		return new Query(new Criteria().andOperator(serviceKontoExistsCriteria, trustLevelNotExistsCriteria));
	}

	Update createUpdate() {
		return new Update().set(KEY_TRUST_LEVEL, DEFAULT_TRUST_LEVEL);
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}
}