/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import java.util.Set;

import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;

@ChangeUnit(id = "2024-09-14 15:00:00 OZG-6641", order = "M012", author = "ebardin", runAlways = true)
public class M012_UpdateBayernIdPostfachType { // NOSONAR

	static final String KEY_HEADER_SERVICE_KONTO_TYPE = "header.serviceKonto.type";
	static final String KEY_EINGANG_SERVICE_KONTO_TYPE = "eingangs." + KEY_HEADER_SERVICE_KONTO_TYPE;
	static final Set<String> POSSIBLE_BAYERN_ID_TYPE = Set.of("BayernId", "BayernID");
	static final String BAYERN_ID_TYPE_NEW = "BAYERN_ID";
	private static final String VORGANG_COLLECTION = "vorgang";

	@Execution
	public void doMigration(MongoTemplate template) {
		updateVorgangHeader(template);
		updateEingangsHeader(template);
	}

	private void updateVorgangHeader(MongoTemplate template) {
		template.updateMulti(createFilterQuery(KEY_HEADER_SERVICE_KONTO_TYPE), createUpdate(KEY_HEADER_SERVICE_KONTO_TYPE), VORGANG_COLLECTION);
	}

	private void updateEingangsHeader(MongoTemplate template) {
		template.updateMulti(createFilterQuery(KEY_EINGANG_SERVICE_KONTO_TYPE), createUpdate("eingangs.$[]." + KEY_HEADER_SERVICE_KONTO_TYPE),
				VORGANG_COLLECTION);
	}

	private Query createFilterQuery(String key) {
		return new Query(Criteria.where(key).in(POSSIBLE_BAYERN_ID_TYPE));
	}

	private Update createUpdate(String key) {
		return new Update().set(key, BAYERN_ID_TYPE_NEW);
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}
}