/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import java.util.List;
import java.util.stream.Stream;

import org.bson.Document;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;

@ChangeUnit(id = "2025-01-17 16:00:00 OZG-7515", order = "M014", author = "freichenbach", runAlways = true)
public class M014_AddItemNameBescheidToPatchAttachedItemCommand { // NOSONAR

	private static final String ID_FIELD = "_id";
	private static final String COMMAND_COLLECTION_NAME = "command";
	private static final String ORDER_FIELD = "order";
	private static final String SEND_BESCHEID_ORDER = "SEND_BESCHEID";
	private static final String PARENT_ID_FIELD = "bodyObject.parentId";
	private static final String PATCH_ATTACHED_ITEM_ORDER = "PATCH_ATTACHED_ITEM";
	private static final String ITEM_NAME_FILED = "bodyObject.itemName";
	private static final String BESCHEID_ITEM_NAME = "Bescheid";

	@Execution
	public void doMigration(MongoOperations mongoOperations) {
		var parentIds = getSendBescheidCommandIds(mongoOperations);
		updateDocuments(mongoOperations, parentIds);
	}

	List<String> getSendBescheidCommandIds(MongoOperations mongoOperations) {
		return getSendBescheidCommands(mongoOperations).map(command -> command.getString(ID_FIELD)).toList();
	}

	private Stream<Document> getSendBescheidCommands(MongoOperations mongoOperations) {
		return mongoOperations.stream(Query.query(Criteria.where(ORDER_FIELD).is(SEND_BESCHEID_ORDER)), Document.class,
				COMMAND_COLLECTION_NAME);
	}

	void updateDocuments(MongoOperations mongoOperations, List<String> parentIds) {
		mongoOperations.updateMulti(createQuery(parentIds), createAddBescheidItemNameUpdate(), COMMAND_COLLECTION_NAME);
	}

	Query createQuery(List<String> parentIds) {
		return new Query().addCriteria(new Criteria().andOperator(
				Criteria.where(PARENT_ID_FIELD).in(parentIds),
				Criteria.where(ORDER_FIELD).is(PATCH_ATTACHED_ITEM_ORDER),
				Criteria.where(ITEM_NAME_FILED).exists(false)));
	}

	private Update createAddBescheidItemNameUpdate() {
		return new Update().set(ITEM_NAME_FILED, BESCHEID_ITEM_NAME);
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}
}