/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.index.TextIndexDefinition;
import org.springframework.stereotype.Component;

import de.ozgcloud.vorgang.vorgang.Vorgang;
import lombok.RequiredArgsConstructor;

@ConditionalOnProperty(prefix = "ozgcloud.mongodbsearch", name = "enabled", havingValue = "true")
@Component
@RequiredArgsConstructor
public class MongodbFullTextSearchIndexInitializer {

	static final String TEXT_INDEX_NAME = "textIndex";
	static final String TEXT_INDEX_LANGUAGE = "german";

	private final MongoOperations mongoOperations;

	@EventListener(ContextRefreshedEvent.class)
	public void initWildcardTextIndex() {
		mongoOperations.indexOps(Vorgang.COLLECTION_NAME).ensureIndex(buildWildcardTextIndexDefinition());
	}

	TextIndexDefinition buildWildcardTextIndexDefinition() {
		return new TextIndexDefinition.TextIndexDefinitionBuilder()
				.named(TEXT_INDEX_NAME)
				.onAllFields()
				.withDefaultLanguage(TEXT_INDEX_LANGUAGE)
				.build();
	}

}
