/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.userconfig;

import java.util.List;

import org.springframework.stereotype.Service;

import de.ozgcloud.user.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;
import de.ozgcloud.user.organisationseinheit.GrpcGetSupportedOrganisationsEinheitenRequest;
import de.ozgcloud.user.organisationseinheit.GrpcOrganisationsEinheit;
import de.ozgcloud.vorgang.callcontext.VorgangManagerClientCallContextAttachingInterceptor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class UserConfigRemoteService {

	@GrpcClient("user-manager")
	private OrganisationsEinheitServiceBlockingStub serviceStub;

	public List<String> getSupportedOrganisationsEinheiten() {
		return getOrganisationsEinheiten().stream().map(GrpcOrganisationsEinheit::getOrganisationsEinheitId)
				.toList();
	}

	List<GrpcOrganisationsEinheit> getOrganisationsEinheiten() {
		return getServiceStub()
				.getSupportedOrganisationsEinheiten(GrpcGetSupportedOrganisationsEinheitenRequest.newBuilder().build())
				.getOrganisationseinheitenList();
	}

	private OrganisationsEinheitServiceBlockingStub getServiceStub() {
		return serviceStub.withInterceptors(new VorgangManagerClientCallContextAttachingInterceptor());
	}
}
