/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.registry;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.vorgang.callcontext.VorgangManagerClientCallContextAttachingInterceptor;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerRegistrationRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.VorgangManagerRegistrationServiceGrpc.VorgangManagerRegistrationServiceBlockingStub;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Log4j2
@Service
@ConditionalOnProperty(prefix = "ozgcloud.zufi-manager", name = { "address" })
class ZufiRemoteService {

	@GrpcClient("zufi-manager")
	private VorgangManagerRegistrationServiceBlockingStub serviceStub;

	public void registerVorgangManager(@NonNull List<String> organistationsEinheitenIds, String vorgangManagerAddress) {
		var result = getServiceStub().register(buildRequest(organistationsEinheitenIds, vorgangManagerAddress));
		if (!result.getSuccess()) {
			LOG.warn("Registration failed: {}", result.getMessage());
		}
	}

	private VorgangManagerRegistrationServiceBlockingStub getServiceStub() {
		return serviceStub.withInterceptors(new VorgangManagerClientCallContextAttachingInterceptor());
	}

	private GrpcVorgangManagerRegistrationRequest buildRequest(List<String> organistationsEinheitenIds, String address) {
		return GrpcVorgangManagerRegistrationRequest.newBuilder().addAllOrganisationsEinheitIds(organistationsEinheitenIds)
				.setServiceAddress(address).build();
	}
}
