/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.statistic;

import org.springframework.beans.factory.annotation.Autowired;

import io.grpc.stub.StreamObserver;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
public class StatisticGrpcService extends StatisticServiceGrpc.StatisticServiceImplBase {

	@Autowired
	private StatisticService statisticService;
	@Autowired
	private CountStatisticMapper countStatisticMapper;
	@Autowired
	private StatisticMapper statisticMapper;

	@Override
	public void countVorgang(GrpcVorgangCountRequest request, StreamObserver<GrpcVorgangCountResponse> responseObserver) {
		var statistic = statisticService.countVorgang(countStatisticMapper.mapFromVorgangCountRequest(request));

		responseObserver.onNext(countStatisticMapper.mapTo(statistic));
		responseObserver.onCompleted();
	}

	@Override
	public void getVorgangStatistic(GrpcVorgangStatisticRequest request, StreamObserver<GrpcVorgangStatisticResponse> responseObserver) {
		var statistic = statisticService.collectStatistic(statisticMapper.mapFromVorgangStatisticRequest(request));

		responseObserver.onNext(statisticMapper.mapToVorgangStatisticResponse(statistic));
		responseObserver.onCompleted();
	}

}