/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.user;

import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import de.ozgcloud.user.userprofile.GrpcDeleteInactiveUserRequest;
import de.ozgcloud.user.userprofile.GrpcFindInactiveUserIdsRequest;
import de.ozgcloud.user.userprofile.GrpcFindInactiveUserIdsResponse;
import de.ozgcloud.user.userprofile.GrpcUserProfileId;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class UserProfileRemoteService {

	@GrpcClient("user-manager")
	private UserProfileServiceBlockingStub userProfileServiceGrpc;

	public Stream<String> findInactiveUserIds() {
		var response = getUserServiceStub().findInactiveUserIds(GrpcFindInactiveUserIdsRequest.newBuilder().build());
		return mapFindInactiveUserIdsResponse(response);
	}

	Stream<String> mapFindInactiveUserIdsResponse(GrpcFindInactiveUserIdsResponse response) {
		return response.getUserProfileIdsList().stream().map(GrpcUserProfileId::getId);
	}

	public void deleteInactiveUser(String userId) {
		getUserServiceStub().deleteInactiveUser(buildDeleteInactiveUsersRequest(userId));
	}

	UserProfileServiceBlockingStub getUserServiceStub() {
		return userProfileServiceGrpc.withInterceptors(new VorgangManagerCallContextAttachingInterceptor());
	}

	GrpcDeleteInactiveUserRequest buildDeleteInactiveUsersRequest(String userId) {
		return GrpcDeleteInactiveUserRequest.newBuilder().setUserId(userId).build();
	}

}
