/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
class VorgangAuthorizationService {

	public boolean authorizeByOrganisationseinheitenId(Vorgang vorgang, FilterCriteria filterCriteria) {
		if (filterCriteria.isFilterByOrganisationseinheitenId()) {
			return matchOrganisationseinheitenId(vorgang, filterCriteria);
		}
		return true;
	}

	private boolean matchOrganisationseinheitenId(Vorgang vorgang, FilterCriteria filterCriteria) {
		var vorgangOrganisationseinheitId = getOrganisationseinheitenId(vorgang);
		if (vorgangOrganisationseinheitId.isPresent()) {
			return filterCriteria.getOrganisationseinheitIds().contains(vorgangOrganisationseinheitId.get());
		}
		return false;
	}

	private Optional<String> getOrganisationseinheitenId(Vorgang vorgang) {
		return Optional.ofNullable(vorgang.getEingangs().get(0))
				.map(Eingang::getZustaendigeStelle)
				.map(ZustaendigeStelle::getOrganisationseinheitenId)
				.map(StringUtils::trimToNull);
	}
}