/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.util.Optional;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

@Mapper
interface VorgangStubMapper {

	@Mapping(target = "deletedAt", ignore = true)
	@Mapping(target = "status", constant = "DELETED")
	@Mapping(target = "header", expression = "java(extractStubHeader(vorgang))")
	@Mapping(target = "eingangs", expression = "java(extractStubEingang(vorgang))")
	VorgangStub fromVorgang(Vorgang vorgang);

	default VorgangHeadStub extractStubHeader(Vorgang vorgang) {
		var builder = VorgangHeadStub.builder();
		getOrganisationsEinheitenId(vorgang).ifPresent(builder::organisationsEinheitId);
		return builder.build();
	}

	default EingangStub extractStubEingang(Vorgang vorgang) {
		var builder = EingangStub.builder();
		getOrganisationsEinheitenId(vorgang)
				.map(orgaId -> ZustaendigeStelleStub.builder().organisationseinheitenId(orgaId).build())
				.ifPresent(builder::zustaendigeStelle);
		return builder.build();

	}

	default Optional<String> getOrganisationsEinheitenId(Vorgang vorgang) {
		try {
			return Optional.ofNullable(vorgang.getEingangs().get(0).getZustaendigeStelle().getOrganisationseinheitenId());
		} catch (NullPointerException | IndexOutOfBoundsException e) {
			return Optional.empty();
		}
	}

	@Mapping(target = "version", ignore = true)
	@Mapping(target = "aktenzeichen", ignore = true)
	@Mapping(target = "assignedTo", ignore = true)
	@Mapping(target = "nummer", ignore = true)
	@Mapping(target = "clientAttributesList", ignore = true)
	@Mapping(target = "formEngineName", ignore = true)
	GrpcVorgangHeader toGrpcVorgangHeader(VorgangStub vorgangStub);
}
