/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.util.UUID;

import lombok.Builder;
import lombok.Getter;

@Getter
@Builder
public class Forwarding {

	public static final String FIELD_STATUS = "status";
	public static final String FIELD_CREATED_AT = "createdAt";
	public static final String FIELD_SENT_AT = "sentAt";
	public static final String FIELD_ERROR_MSG = "errorMessage";

	enum Status {
		NEW, SENT, SUCCESSFULL, SEND_ERROR, FAILED
	}

	@Builder.Default
	private String id = UUID.randomUUID().toString();
	private String vorgangId;

	private Status status;

	private String zustaendigeStelle;

	@Builder.Default
	private ZonedDateTime createdAt = ZonedDateTime.now(ZoneOffset.UTC).withNano(0); // MongoDB cannot persist nanos
	private String createdBy;
	private String createdByName;
	private String createdByCommand;

	private ZonedDateTime sentAt;

	private String errorMessage;

}
