/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import static org.springframework.data.mongodb.core.query.Criteria.*;
import static org.springframework.data.mongodb.core.query.Query.*;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

import de.ozgcloud.vorgang.vorgang.Vorgang;

@Repository
class ForwardingRepository {

	private static final String FORWARDING_FIELD = "forwardings";

	@Autowired
	private MongoOperations mongoOperations;
	@Autowired
	private MongoTemplate template;

	public Forwarding save(String vorgangId, Forwarding forwarding) {
		mongoOperations.updateFirst(queryVorgangById(vorgangId),
				new Update().addToSet(FORWARDING_FIELD, forwarding),
				Vorgang.class);

		return forwarding;
	}

	public void patch(String forwardingId, Map<String, Object> patch) {
		Update update = new Update();
		patch.forEach((key, value) -> update.set(FORWARDING_FIELD + ".$." + key, value));

		mongoOperations.updateFirst(queryForwardingById(forwardingId),
				update,
				Vorgang.class);
	}

	private Query queryVorgangById(String vorgangId) {
		return query(where(Vorgang.MONGODB_FIELDNAME_ID).is(vorgangId));
	}

	private Query queryForwardingById(String forwardingId) {
		return query(where(FORWARDING_FIELD + "._id").is(forwardingId));
	}

	public List<Forwarding> findByVorgangId(String vorgangId) {
		var aggregation = Aggregation.newAggregation(
				Aggregation.match(new Criteria(Vorgang.MONGODB_FIELDNAME_ID).is(vorgangId)),
				Aggregation.project(FORWARDING_FIELD),
				Aggregation.unwind(FORWARDING_FIELD),
				Aggregation.replaceRoot(FORWARDING_FIELD),
				Aggregation.sort(Sort.by(Forwarding.FIELD_CREATED_AT)));

		var result = template.aggregate(aggregation, Vorgang.class, Forwarding.class);
		return result.getMappedResults();
	}
}
