/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.test.annotation.DirtiesContext;

import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.vorgang.VorgangManagerServerApplication;
import de.ozgcloud.vorgang.callcontext.CallContextUser;
import de.ozgcloud.vorgang.callcontext.CurrentUserService;
import de.ozgcloud.vorgang.servicekonto.PostfachAddressTestFactory;
import de.ozgcloud.vorgang.servicekonto.ServiceKontoTestFactory;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangHeader;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import net.devh.boot.grpc.client.inject.GrpcClient;

@SpringBootTest(classes = { VorgangManagerServerApplication.class }, properties = {
		"grpc.server.inProcessName=test",
		"grpc.client.inProcess.address=in-process:test"
})
@DirtiesContext
@DataITCase
class FindVorgangIteratorITCase {

	@GrpcClient("inProcess")
	private VorgangServiceBlockingStub vorgangServiceStub;

	@Autowired
	private MongoOperations mongoOperations;

	@MockBean
	private CurrentUserService currentUserService;
	@Mock
	private CallContextUser callContextUser;

	private FindVorgangIterator findVorgangIterator;

	@BeforeEach
	void init() {
		mongoOperations.dropCollection(Vorgang.COLLECTION_NAME);
		when(currentUserService.getUser()).thenReturn(callContextUser);
		findVorgangIterator = new FindVorgangIterator(vorgangServiceStub, ServiceKontoTestFactory.TYPE,
				PostfachAddressTestFactory.STRING_BASED_IDENTIFIER_POSTFACH_ID_VALUE);
	}

	@Test
	void shouldFindVorgangs() {
		var vorgang = mongoOperations.save(VorgangTestFactory.createBuilder().id(null).build());

		var hasNext = findVorgangIterator.hasNext();
		assertThat(hasNext).isTrue();

		var result = findVorgangIterator.next();
		assertThat(result.getId()).isEqualTo(vorgang.getId());
	}

	@Test
	void shouldLoadAllVorgangs() {
		IntStream.range(0, FindVorgangIterator.LIMIT + 1).forEach(i -> mongoOperations.save(VorgangTestFactory.createBuilder().id(null).build()));

		var result = loadAllVorgangs();

		assertThat(result).hasSize(FindVorgangIterator.LIMIT + 1);
	}

	private List<GrpcVorgangHeader> loadAllVorgangs() {
		var result = new ArrayList<GrpcVorgangHeader>();
		while (findVorgangIterator.hasNext()) {
			result.add(findVorgangIterator.next());
		}
		return result;
	}

}