/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.clientattribute;

import static de.ozgcloud.vorgang.clientattribute.ClientAttributeTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import jakarta.validation.ConstraintValidatorContext;

class ClientAttributeHasValueValidatorTest {

	@InjectMocks
	private ClientAttributeHasValueValidator validator;

	@Mock
	private ConstraintValidatorContext context;

	@Nested
	@DisplayName("should be valid")
	class TestValid {

		@Test
		void withIntValue() {
			var result = validator.isValid(ClientAttributeTestFactory.createBuilder().intValue(Optional.of(INT_VALUE)).build(), context);

			assertThat(result).isTrue();
		}

		@Test
		void withStringValue() {
			var result = validator.isValid(ClientAttributeTestFactory.createBuilder()
					.intValue(Optional.empty()).stringValue(Optional.of(STR_VALUE)).build(), context);

			assertThat(result).isTrue();
		}

		@Test
		void withDoubleValue() {
			var result = validator.isValid(ClientAttributeTestFactory.createBuilder()
					.intValue(Optional.empty()).doubleValue(Optional.of(DOUBLE_VALUE)).build(), context);

			assertThat(result).isTrue();
		}

		@Test
		void withBooleanValue() {
			var result = validator.isValid(ClientAttributeTestFactory.createBuilder()
					.intValue(Optional.empty()).boolValue(Optional.of(BOOL_VALUE)).build(), context);

			assertThat(result).isTrue();
		}
	}

	@Nested
	@DisplayName("should be invalid")
	class TestInValid {
		@Test
		void withoutAnyValue() {
			var result = validator.isValid(ClientAttributeTestFactory.createBuilder()
					.intValue(Optional.empty()).build(), context);

			assertThat(result).isFalse();
		}

		@Test
		void withMoreThenOneValue() {
			var result = validator.isValid(ClientAttributeTestFactory.createBuilder()
					.intValue(Optional.of(INT_VALUE)).boolValue(Optional.of(BOOL_VALUE)).build(), context);

			assertThat(result).isFalse();
		}

	}
}
