/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.command.CommandExecutedEvent;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.command.CommandRevokeFailedEvent;
import de.ozgcloud.command.CommandRevokedEvent;

class CommandEventListenerTest {

	@Spy
	@InjectMocks
	private CommandEventListener listener;

	@Mock
	private CommandService commandService;

	@Nested
	class TestCommandExecutedEventListener {

		private final CommandExecutedEvent commandExecutedEvent = CommandExecutedEventTestFactory.createFor(CommandTestFactory.create(),
				CommandTestFactory.CREATED_RESOURCE);

		@Test
		void shouldCallSetStatusFinished() {
			listener.setStatusFinished(commandExecutedEvent);

			verify(commandService).setCommandFinished(CommandTestFactory.ID, CommandTestFactory.CREATED_RESOURCE);
		}

	}

	@Nested
	class TestCommandRevokeFailedEventListener {

		@Test
		void shouldSetStatusFinished() {
			listener.setStatusFinished(new CommandRevokeFailedEvent(CommandTestFactory.ID, "test Error"));

			verify(commandService).setCommandFinished(CommandTestFactory.ID);
		}
	}

	@Nested
	class TestCommandFailedEventListener {
		private static final String ERROR_MESSAGE = "test Error";

		private final CommandFailedEvent commandFailedEvent = new CommandFailedEvent(CommandTestFactory.ID, ERROR_MESSAGE);

		@Test
		void shouldSetStatusError() {
			listener.setStatusError(commandFailedEvent);

			verify(commandService).setCommandError(CommandTestFactory.ID, ERROR_MESSAGE);
		}

	}

	@Nested
	class TestCommandRevokedEventListener {
		@Test
		void shouldSetStatusRevoked() {
			listener.setCommandRevoked(new CommandRevokedEvent(CommandTestFactory.create()));

			verify(commandService).setCommandRevoked(CommandTestFactory.ID);
		}
	}

}
