/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

class GrpcAddSubCommandsRequestMapperTest {

	@Spy
	@InjectMocks
	private GrpcAddSubCommandsRequestMapper mapper = Mappers.getMapper(GrpcAddSubCommandsRequestMapper.class);

	@Mock
	private GrpcCreateCommandMapper subCommandMapper;

	@Nested
	class TestFrom {

		@Test
		void shouldSetParentId() {
			var result = mapFromGrpcAddSubCommandsRequest();

			assertThat(result.getParentId()).isEqualTo(GrpcAddSubCommandsRequestTestFactory.PARENT_ID);
		}

		@Test
		void shouldSetExecutionMode() {
			var result = mapFromGrpcAddSubCommandsRequest();

			assertThat(result.getExecutionMode()).isEqualTo(SubCommandExecutionMode.PARALLEL);
		}

		@Test
		void shouldSetCompletedIfSubsCompleted() {
			var result = mapFromGrpcAddSubCommandsRequest();

			assertThat(result.isCompletedIfSubsCompleted()).isTrue();
		}

		@Test
		void shouldCallSubCommandMapper() {
			var subCommand = GrpcCreateCommandTestFactory.create();
			var request = GrpcAddSubCommandsRequestTestFactory.createBuilder().clearSubCommands().addSubCommands(subCommand).build();

			mapper.from(request);

			verify(subCommandMapper).from(subCommand);
		}

		@Test
		void shouldSetCreateSubCommandRequests() {
			var subCommand = CreateSubCommandRequestTestFactory.create();
			when(subCommandMapper.from(any())).thenReturn(subCommand);

			var result = mapFromGrpcAddSubCommandsRequest();

			assertThat(result.getSubCommands()).containsExactly(subCommand);
		}

		private CreateSubCommandsRequest mapFromGrpcAddSubCommandsRequest() {
			return mapper.from(GrpcAddSubCommandsRequestTestFactory.create());
		}
	}

	@Nested
	class TestMapExecutionMode {

		@DisplayName("should return PARALLEL")
		@ParameterizedTest(name = "for value \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { "PARALLEL", "some_value" })
		void shouldMapExecutionMode(String value) {
			var result = mapper.mapExecutionMode(value);

			assertThat(result).isEqualTo(SubCommandExecutionMode.PARALLEL);
		}
	}

}