/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import java.util.List;

import org.bson.Document;
import org.bson.types.ObjectId;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Component;

import com.mongodb.client.FindIterable;

@Component
class MigrationDbTestUtils {

	public static final String COMMAND_COLLECTION = "command";
	public static final String GRID_FS_FILE_COLLECTION = "fs.files";
	public static final String VORGANG_COLLECTION = "vorgang";
	public static final String VORGANG_ATTACHED_ITEM_COLLECTION = "vorgangAttachedItem";

	@Autowired
	private MongoTemplate template;
	@Autowired
	private MongoOperations operations;

	// Command
	public Document saveCommand(Document doc) {
		return template.save(doc, COMMAND_COLLECTION);
	}

	public Document getCommand(String id) {
		return template.findById(id, Document.class, COMMAND_COLLECTION);
	}

	public void dropCommandCollection() {
		template.dropCollection(COMMAND_COLLECTION);
	}

	// GridFs
	public Document saveGridFsFile(Document doc) {
		return template.save(doc, GRID_FS_FILE_COLLECTION);
	}

	public List<Document> findGridFsFile(Query query) {
		return template.find(query, Document.class, GRID_FS_FILE_COLLECTION);
	}

	public void dropGridFsFilesCollection() {
		template.dropCollection(GRID_FS_FILE_COLLECTION);
	}

	public FindIterable<Document> findGridFsFile(Document doc) {
		return operations.getCollection(GRID_FS_FILE_COLLECTION).find(doc);
	}

	// Vorgang
	public Document saveVorgang(Document doc) {
		return template.save(doc, VORGANG_COLLECTION);
	}

	public Document getVorgang(ObjectId id) {
		return template.findById(id, Document.class, VORGANG_COLLECTION);
	}

	public List<Document> findVorgang(Query query) {
		return template.find(query, Document.class, VORGANG_COLLECTION);
	}

	public void dropVorgangCollection() {
		template.dropCollection(VORGANG_COLLECTION);
	}

	// VorgangAttachedItem
	public Document saveVorgangAttachedItem(Document doc) {
		return template.save(doc, VORGANG_ATTACHED_ITEM_COLLECTION);
	}

	public Document getVorgangAttachedItem(ObjectId id) {
		return template.findById(id, Document.class, VORGANG_ATTACHED_ITEM_COLLECTION);
	}

	public List<Document> findVorgangAttachedItem(Query query) {
		return template.find(query, Document.class, VORGANG_ATTACHED_ITEM_COLLECTION);
	}

	public void dropVorgangAttachedItemCollection() {
		template.dropCollection(VORGANG_ATTACHED_ITEM_COLLECTION);
	}
}