/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.operator;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.vorgang.common.operator.FieldPathProcessor;
import de.ozgcloud.vorgang.common.operator.UnsupportedEntityException;

class FieldPathProcessorTest {

	@Nested
	class TestProcessToCriteriaPath {

		private final static String VORGANG_FIELD_PATH = "Vorgang.long.path";
		private final static String CLIENT_ATTRIBUTE_FIELD_PATH = "ClientAttribute.clientName.frist";

		@Nested
		class GetCriteriaEntity {
			@Test
			void shouldReturnVorgang() {
				var entity = FieldPathProcessor.getCriteriaEntity(VORGANG_FIELD_PATH);

				assertThat(entity).isEqualTo(FieldPathProcessor.CRITERIA_PATH_VORGANG);
			}

			@Test
			void shouldThrowExcptionForUnkownEntity() {
				assertThatThrownBy(() -> FieldPathProcessor.getCriteriaEntity("stupid.path"))
						.isInstanceOf(UnsupportedEntityException.class);

			}
		}

		@Nested
		class GetPath {

			@Test
			void shouldReturnPathWithoutEntity() {
				var path = FieldPathProcessor.getPath(VORGANG_FIELD_PATH);

				assertThat(path).isEqualTo("long.path");
			}

			@Test
			void shouldAddValueSuffix() {
				var path = FieldPathProcessor.getPath(CLIENT_ATTRIBUTE_FIELD_PATH);

				assertThat(path).endsWith(".value");
			}
		}

		@Test
		void shouldReturnCriteriaWithoutEntity() {
			var criteria = FieldPathProcessor.processToCriteriaPath(VORGANG_FIELD_PATH);

			assertThat(criteria).isEqualTo("long.path");
		}

		@Test
		void shouldReturnCriteriaWithEntity() {
			var criteria = FieldPathProcessor.processToCriteriaPath(CLIENT_ATTRIBUTE_FIELD_PATH);

			assertThat(criteria).isEqualTo("clientAttributes.clientName.frist.value");
		}
	}

}
