/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.index.IndexOperations;
import org.springframework.data.mongodb.core.index.TextIndexDefinition;

import de.ozgcloud.vorgang.vorgang.Vorgang;

class MongodbFullTextSearchIndexInitializerTest {

	@Spy
	@InjectMocks
	private MongodbFullTextSearchIndexInitializer indexInitializer;

	@Mock
	private MongoOperations mongoOperations;

	@Nested
	class TestInitWildcardTextIndex {

		@Mock
		private IndexOperations indexOperations;
		@Mock
		private TextIndexDefinition textIndexDefinition;

		@BeforeEach
		void init() {
			when(mongoOperations.indexOps(anyString())).thenReturn(indexOperations);
		}

		@Test
		void shouldCallEnsureIndex() {
			indexInitializer.initWildcardTextIndex();

			verify(mongoOperations).indexOps(Vorgang.COLLECTION_NAME);
		}

		@Test
		void shouldCallBuildWildcardTextIndexDefinition() {
			indexInitializer.initWildcardTextIndex();

			verify(indexInitializer).buildWildcardTextIndexDefinition();
		}

		@Test
		void shouldCallEnsureIndexWithBuildWildcardTextIndexDefinition() {
			doReturn(textIndexDefinition).when(indexInitializer).buildWildcardTextIndexDefinition();

			indexInitializer.initWildcardTextIndex();

			verify(indexOperations).ensureIndex(textIndexDefinition);
		}
	}

	@Nested
	class TestBuildWildcardTextIndexDefinition {

		@Test
		void shouldSetIndexName() {
			var result = indexInitializer.buildWildcardTextIndexDefinition();

			assertThat(result.getIndexOptions()).containsEntry("name", MongodbFullTextSearchIndexInitializer.TEXT_INDEX_NAME);
		}

		@Test
		void shouldSetDefaultLanguage() {
			var result = indexInitializer.buildWildcardTextIndexDefinition();

			assertThat(result.getIndexOptions()).containsEntry("default_language", MongodbFullTextSearchIndexInitializer.TEXT_INDEX_LANGUAGE);
		}

		@Test
		void shouldCreateWildcardTextIndex() {
			var result = indexInitializer.buildWildcardTextIndexDefinition();

			assertThat(result.getIndexKeys()).containsEntry("$**", "text");
		}
	}
}