/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.security;

import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.bson.Document;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.data.mongodb.core.MongoTemplate;

import de.ozgcloud.vorgang.vorgang.ZustaendigeStelleTestFactory;

class PolicyRepositoryTest {

	@InjectMocks
	private PolicyRepository repository;

	@Mock
	private MongoTemplate mongoTemplate;

	@Nested
	class TestEvaluateOrganisationseinheitId {

		@Nested
		class TestMatch {

			@Test
			void shouldEvaluateVorgang() {
				var document = new Document(PolicyRepository.FIELD_ORGANISATIONSEINHEIT_ID,
						List.of(List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID)));

				var result = repository.evaluateOrganisationseinheitId(document, List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID));

				assertThat(result).isTrue();
			}

			@Test
			void shouldEvaluateVorgangStub() {
				var document = new Document(PolicyRepository.FIELD_ORGANISATIONSEINHEIT_ID,
						List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID));

				var result = repository.evaluateOrganisationseinheitId(document, List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID));

				assertThat(result).isTrue();
			}
		}

		@Test
		void  shouldReturnFalseIfNoMatch() {
			var document = new Document(PolicyRepository.FIELD_ORGANISATIONSEINHEIT_ID,
					List.of(List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID)));

			var result = repository.evaluateOrganisationseinheitId(document, List.of("otherId"));

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnFalseIfEmpty() {
			var document = new Document(PolicyRepository.FIELD_ORGANISATIONSEINHEIT_ID, List.of());

			var result = repository.evaluateOrganisationseinheitId(document, List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID));

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnFalseIfNestedEmpty() {
			var document = new Document(PolicyRepository.FIELD_ORGANISATIONSEINHEIT_ID, List.of(List.of()));

			var result = repository.evaluateOrganisationseinheitId(document, List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID));

			assertThat(result).isFalse();
		}
	}
}