/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.userconfig;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;
import de.ozgcloud.vorgang.callcontext.VorgangManagerClientCallContextAttachingInterceptor;

class UserConfigRemoteServiceTest {
	@Spy
	@InjectMocks
	private UserConfigRemoteService remoteService;

	@Mock
	private OrganisationsEinheitServiceBlockingStub stub;

	@DisplayName("Test getting Organsiationseinheiten Ids of the users")
	@Nested
	class TestGettingOrganisationsEinheitenIds {
		@BeforeEach
		void init() {
			when(stub.getSupportedOrganisationsEinheiten(any()))
					.thenReturn(GrpcGetSupportedOrganisationEinheitenResponseTestFactory.create());
			when(stub.withInterceptors(any())).thenReturn(stub);
		}

		@Test
		void shouldGetOrganisationsEinheiten() {
			var organisationsEinheiten = remoteService.getSupportedOrganisationsEinheiten();

			assertThat(organisationsEinheiten).hasSize(1)
					.contains(GrpcGetSupportedOrganisationEinheitenResponseTestFactory.ORGANISATIONS_EINHEIT_ID);
		}

		@Test
		void shouldUseInterceptor() {
			remoteService.getSupportedOrganisationsEinheiten();

			verify(stub).withInterceptors(any(VorgangManagerClientCallContextAttachingInterceptor.class));
		}
	}
}
