/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.util.Random;
import java.util.UUID;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.vorgang.callcontext.CallContextTestFactory;
import de.ozgcloud.vorgang.callcontext.UserTestFactory;
import de.ozgcloud.vorgang.vorgang.IncomingFileTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;

public class OzgCloudFileTestFactory {

	public static final String ID_STR = UUID.randomUUID().toString();
	public static final FileId ID = FileId.from(ID_STR);
	public static final long VERSION = new Random().nextLong();
	public static final String NAME = LoremIpsum.getInstance().getWords(1);
	public static final String CONTENT_TYPE = IncomingFileTestFactory.CONTENT_TYPE_STR;
	public static final String CONTENT_ID = UUID.randomUUID().toString();
	public static final String CLIENT = CallContextTestFactory.CLIENT;
	public static final String FIELD_NAME = LoremIpsum.getInstance().getWords(1);
	public static final String CREATED_BY = UserTestFactory.ID;

	public static final byte[] CONTENT = "juhu, ein bild".getBytes();
	public static final long SIZE = CONTENT.length;

	public static OzgCloudFile create() {
		return createBuilder().build();
	}

	public static OzgCloudFile.OzgCloudFileBuilder createBuilder() {
		return OzgCloudFile.builder()
				.id(ID_STR)
				.version(VERSION)
				.vorgangId(VorgangTestFactory.ID)
				.name(NAME)
				.contentType(CONTENT_TYPE)
				.size(SIZE)
				.contentId(CONTENT_ID)
				.client(CLIENT)
				.createdBy(CREATED_BY)
				.fieldName(FIELD_NAME);
	}

	public static byte[] createContentInMiB(int sizeInMiB) {
		return createContentInByte(sizeInMiB * 1_000_000);
	}

	public static byte[] createContentInByte(int fileSize) {
		Random r = new Random();
		byte[] chunk = new byte[fileSize];
		for (int i = 0; i < chunk.length; i++) {
			chunk[i] = (byte) (r.nextInt('z' - 'a') + 'a');
		}
		return chunk;
	}
}
