/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.servicekonto;

import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;

class ServiceKontoTest {

	@InjectMocks
	private ServiceKonto serviceKonto;

	@Test
	void shouldCallValidateTrustLevel() {
		var builder = spy(ServiceKonto.builder());

		builder.trustLevel(ServiceKontoTestFactory.TRUST_LEVEL);

		verify(builder).validateTrustLevel(ServiceKontoTestFactory.TRUST_LEVEL);
	}

	@DisplayName("should validate trust level when")
	@ParameterizedTest(name = "value is \"{0}\"")
	@NullAndEmptySource
	@ValueSource(strings = { "STORK-QAA-Level-1", "STORK-QAA-Level-2", "STORK-QAA-Level-3", "STORK-QAA-Level-4" })
	void shouldValidateTrustLevelWhen(String trustLevel) {
		var builder = spy(ServiceKonto.builder());

		builder.validateTrustLevel(trustLevel);

		verify(builder, never()).logError(trustLevel);
	}

	@DisplayName("should log error when trust level is not expected")
	@ParameterizedTest(name = "value is \"{0}\"")
	@ValueSource(strings = { "trust-me", " " })
	void shouldLogErrorWhenTrustLevelIsNotExpected(String trustLevel) {
		var builder = spy(ServiceKonto.builder());

		builder.validateTrustLevel(trustLevel);

		verify(builder).logError(trustLevel);
	}
}
