/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.statistic;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import io.grpc.stub.StreamObserver;

class StatisticGrpcServiceTest {

	@Spy
	@InjectMocks
	private StatisticGrpcService grpcStatisticService;

	@Mock
	private StatisticService statisticService;
	@Mock
	private CountStatisticMapper countStatisticMapper;
	@Mock
	private StatisticMapper statisticMapper;

	@DisplayName("Count vorgang")
	@Nested
	class TestCountVorgang {

		@Mock
		private GrpcVorgangCountRequest grpcRequest;
		@Mock
		private StreamObserver<GrpcVorgangCountResponse> responseStreamObserver;
		@Mock
		private GrpcVorgangCountResponse grpcResponse;

		private Statistic statistic = StatisticTestFactory.create();
		private VorgangCountRequest request = VorgangCountRequest.builder().build();

		@BeforeEach
		void setUp() {
			when(statisticService.countVorgang(any())).thenReturn(statistic);
			when(countStatisticMapper.mapFromVorgangCountRequest(any())).thenReturn(request);
		}

		@Test
		void shouldCallMapFrom() {
			grpcStatisticService.countVorgang(grpcRequest, responseStreamObserver);

			verify(countStatisticMapper).mapFromVorgangCountRequest(grpcRequest);
		}

		@Test
		void shouldCallService() {
			grpcStatisticService.countVorgang(grpcRequest, responseStreamObserver);

			verify(statisticService).countVorgang(request);
		}

		@Test
		void shouldCallMapTo() {
			grpcStatisticService.countVorgang(grpcRequest, responseStreamObserver);

			verify(countStatisticMapper).mapTo(statistic);
		}

		@Test
		void shouldSetResponse() {
			when(countStatisticMapper.mapTo(any())).thenReturn(grpcResponse);

			grpcStatisticService.countVorgang(grpcRequest, responseStreamObserver);

			verify(responseStreamObserver).onNext(grpcResponse);
		}

		@Test
		void shouldCompleteResponse() {
			grpcStatisticService.countVorgang(grpcRequest, responseStreamObserver);

			verify(responseStreamObserver).onCompleted();
		}
	}

	@Nested
	class TestGetVorgangStatistic {

		@Mock
		private StreamObserver<GrpcVorgangStatisticResponse> responseObserver;

		private GrpcVorgangStatisticRequest request = GrpcVorgangStatisticRequestTestFactory.create();
		private GrpcVorgangStatisticResponse response = GrpcVorgangStatisticResponseTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(response).when(statisticMapper).mapToVorgangStatisticResponse(any());
		}

		@Test
		void shouldSetResponse() {
			grpcStatisticService.getVorgangStatistic(request, responseObserver);

			verify(responseObserver).onNext(response);
		}

		@Test
		void shouldCompleteResponse() {
			grpcStatisticService.getVorgangStatistic(request, responseObserver);

			verify(responseObserver).onCompleted();
		}

		@Test
		void shouldCallMapFromStatisticRequest() {
			grpcStatisticService.getVorgangStatistic(request, responseObserver);

			verify(statisticMapper).mapFromVorgangStatisticRequest(request);
		}

		@Test
		void shouldCallStatisticService() {
			var statisticRequests = List.of(VorgangStatisticRequestTestFactory.create());
			when(statisticMapper.mapFromVorgangStatisticRequest(any())).thenReturn(statisticRequests);

			grpcStatisticService.getVorgangStatistic(request, responseObserver);

			verify(statisticService).collectStatistic(statisticRequests);
		}

		@Test
		void shouldCallMapToStatisticRequest() {
			var statistic = StatisticTestFactory.create();
			when(statisticService.collectStatistic(any())).thenReturn(statistic);

			grpcStatisticService.getVorgangStatistic(request, responseObserver);

			verify(statisticMapper).mapToVorgangStatisticResponse(statistic);
		}
	}
}