/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.vorgang.command.CommandTestFactory;
import de.ozgcloud.vorgang.vorgang.Lock.LockBuilder;

public class LockTestFactory {

	public static final String CLIENT_NAME = CommandTestFactory.CREATED_BY_CLIENT;
	public static final String REASON = LoremIpsum.getInstance().getWords(5);
	public static final String LOCKED_SINCE_STR = ZonedDateTime.now().withNano(0).format(DateTimeFormatter.ISO_INSTANT);
	public static final ZonedDateTime LOCKED_SINCE = ZonedDateTime.parse(LOCKED_SINCE_STR);

	public static Lock create() {
		return createBuilder().build();
	}

	public static LockBuilder createBuilder() {
		return Lock.builder()
				.clientName(CLIENT_NAME)
				.lockedSince(LOCKED_SINCE)
				.reason(REASON);
	}

	public static Map<String, Object> asMap() {
		return Map.of(
				Lock.FIELD_CLIENT_NAME, CLIENT_NAME,
				Lock.FIELD_LOCKED_SINCE, LOCKED_SINCE,
				Lock.FIELD_REASON, REASON);
	}
}
