/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;

class VorgangStubMapperTest {

	@InjectMocks
	private VorgangStubMapper mapper = Mappers.getMapper(VorgangStubMapper.class);

	@Nested
	class TestFormVorgang {
		@Test
		void shouldMapFields() {
			var stub = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(stub).usingRecursiveComparison()
					.ignoringFields("deletedAt", "status")
					.isEqualTo(VorgangStubTestFactory.create());
		}

		@Test
		void shouldHaveStatusDELETED() {
			var stub = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(stub.getStatus()).isEqualTo(Vorgang.Status.DELETED);
		}

		@Test
		void shouldHaveDeletedAtOnNow() {
			var stub = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(stub.getDeletedAt()).isCloseTo(ZonedDateTime.now(), byLessThan(500, ChronoUnit.MILLIS));
		}
	}

	@Nested
	class TestToGrpcVorgangHeader {

		@Test
		void shouldMapFields() {
			var vorgangStub = VorgangStubTestFactory.createBuilder().status(Vorgang.Status.DELETED).build();
			var expectedGrpcVorgangHeader = GrpcVorgangHeader.newBuilder()
					.setId(VorgangStubTestFactory.ID)
					.setName(VorgangStubTestFactory.NAME)
					.setCreatedAt(VorgangStubTestFactory.CREATED_AT_STR)
					.setStatus(Vorgang.Status.DELETED.name())
					.build();

			var mappedGrpcVorgangHeader = mapper.toGrpcVorgangHeader(vorgangStub);

			assertThat(mappedGrpcVorgangHeader).isEqualTo(expectedGrpcVorgangHeader);
		}
	}
}
