/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.time.ZonedDateTime;

import org.bson.types.ObjectId;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.vorgang.callcontext.UserTestFactory;
import de.ozgcloud.vorgang.clientattribute.ClientAttributesMap;
import de.ozgcloud.vorgang.clientattribute.ClientAttributesMapTestFactory;
import de.ozgcloud.vorgang.vorgang.Vorgang.Status;

public class VorgangTestFactory {

	public static final String ID = new ObjectId().toHexString();
	public static final String NAME = LoremIpsum.getInstance().getName();
	public static final long VERSION = 42L;
	public static final Status STATUS = Status.NEU;

	public static final String INITIAL_DATE = "2021-01-10";
	public static final String INITIAL_TIME = "10:30:00";
	public static final String INITIAL_DATE_STR = "%sT%sZ".formatted(INITIAL_DATE, INITIAL_TIME);
	public static final String CREATED_AT_STR = INITIAL_DATE_STR;
	public static final ZonedDateTime CREATED_AT = ZonedDateTime.parse(CREATED_AT_STR);

	public static final String VORGANG_NUMMER = "VOR-GANG-NUMMER-1";
	public static final String AKTENZEICHEN = LoremIpsum.getInstance().getWords(1);
	public static final boolean IN_CREATION = false;

	public static final Eingang EINGANG = EingangTestFactory.create();

	public static final ClientAttributesMap CLIENT_ATTRIBUTES_MAP = ClientAttributesMapTestFactory.create();
	public static final String FORM_ENGINE_NAME = "TTFE";

	public static Vorgang create() {
		return createBuilder().build();
	}

	public static Vorgang createWithOrganisationEinheitId(String organisationEinheitId) {
		return VorgangTestFactory.createBuilder().clearEingangs()
				.eingang(EingangTestFactory.createBuilder()
						.zustaendigeStelle(ZustaendigeStelleTestFactory.createBuilder().organisationseinheitenId(organisationEinheitId).build())
						.build())
				.build();
	}

	public static Vorgang.VorgangBuilder createBuilder() {
		return Vorgang.builder()
				.id(ID)
				.name(NAME)
				.aktenzeichen(AKTENZEICHEN)
				.nummer(VORGANG_NUMMER)
				.status(STATUS)
				.createdAt(CREATED_AT)
				.header(VorgangHeadTestFactory.createBuilder().lock(null).build())
				.eingang(EINGANG)
				.version(VERSION)
				.assignedTo(UserTestFactory.ID)
				.inCreation(IN_CREATION)
				.clientAttributes(CLIENT_ATTRIBUTES_MAP);
	}
}