/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import static org.awaitility.Awaitility.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.concurrent.TimeUnit;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.mail.javamail.JavaMailSender;

import de.ozgcloud.command.Command;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.nachrichten.email.MailSentEventTestFactory;
import de.ozgcloud.vorgang.command.CommandCreatedEventTestFactory;
import de.ozgcloud.vorgang.command.CommandTestFactory;
import de.ozgcloud.vorgang.command.Order;
import de.ozgcloud.vorgang.vorgang.VorgangService;

@ITCase
class ForwardingEventListenerITCase {

	@Autowired
	private ApplicationEventPublisher publisher;

	@MockBean
	private ForwardingService service;
	@MockBean
	private JavaMailSender mailSender;
	@MockBean
	private VorgangService vorgangService;

	@Nested
	class TestForwardVorgang {

		@Test
		void shouldCallForwardingService() {
			var command = CommandTestFactory.createBuilder().order(Order.REDIRECT_VORGANG.name()).build();

			publisher.publishEvent(CommandCreatedEventTestFactory.create(command));

			await().atMost(5, TimeUnit.SECONDS).untilAsserted(() -> verify(service).forwardByCommand(command));
		}

	}

	@Nested
	class TestMailSentEvent {
		@Test
		void shouldCallForwardingService() {
			Forwarding forwarding = ForwardingTestFactory.create();

			publisher.publishEvent(MailSentEventTestFactory.create(forwarding));

			await().atMost(5, TimeUnit.SECONDS).untilAsserted(() -> verify(service).onRedirectMailSent(forwarding));
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "REDIRECT_VORGANG" })
		void shouldNOTCallServiceOnAnyOtherOrder(Order order) {
			Command command = CommandTestFactory.createBuilder().order(order.name()).build();

			publisher.publishEvent(MailSentEventTestFactory.create(command));

			verify(service, never()).onRedirectMailSent(any());
		}
	}

	@Nested
	class CommandCreatedEvent {

		@Test
		void shouldCallForwardingService() {
			var command = CommandTestFactory.createBuilder().order(Order.FORWARD_SUCCESSFULL.name()).build();

			publisher.publishEvent(CommandCreatedEventTestFactory.create(command));

			await().atMost(5, TimeUnit.SECONDS).untilAsserted(() -> verify(service).markAsSuccessfull(command));
		}
	}

}
