/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.ConcurrentModificationException;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.nachrichten.email.MailSentEventTestFactory;
import de.ozgcloud.vorgang.command.CommandCreatedEventTestFactory;
import de.ozgcloud.vorgang.command.CommandTestFactory;
import de.ozgcloud.vorgang.command.Order;

class ForwardingEventListenerTest {

	@InjectMocks // NOSONAR
	private ForwardingEventListener listener;

	@Mock
	private ForwardingService forwardingService;
	@Mock
	private ApplicationEventPublisher publisher;

	@Nested
	class onForwardOrderCommand {

		@Test
		void shouldCallService() {
			listener.onForwardOrder(CommandCreatedEventTestFactory.create());

			verify(forwardingService).forwardByCommand(notNull());
		}

		@Test
		void shouldPublishFailedEventOnError() {
			when(forwardingService.forwardByCommand(any())).thenThrow(ConcurrentModificationException.class);
			
			listener.onForwardOrder(CommandCreatedEventTestFactory.create());
			
			verify(publisher).publishEvent(any(CommandFailedEvent.class));
		}
	}

	@Nested
	class onMailSentEvent {

		private Forwarding forwarding = ForwardingTestFactory.create();

		@Test
		void shouldCallRedirectService() {
			listener.triggerRedirectMailSent(MailSentEventTestFactory.create(forwarding));

			verify(forwardingService).onRedirectMailSent(forwarding);
		}

	}

	@Nested
	class TestMarkAsSuccessfull {

		@Test
		void shouldCallService() {
			Command command = CommandTestFactory.createBuilder().order(Order.FORWARD_SUCCESSFULL.name()).build();

			listener.markAsSuccessfull(CommandCreatedEventTestFactory.create(command));

			verify(forwardingService).markAsSuccessfull(command);
		}

		@Test
		void shouldPublishFailedEventOnError() {
			doThrow(RuntimeException.class).when(forwardingService).markAsSuccessfull(any());

			listener.markAsSuccessfull(CommandCreatedEventTestFactory.create());

			verify(publisher).publishEvent(any(CommandFailedEvent.class));
		}

	}

	@Nested
	class TestMarkAsFailed {

		@Test
		void shouldCallService() {
			Command command = CommandTestFactory.createBuilder().order(Order.FORWARD_FAILED.name()).build();

			listener.markAsFailed(CommandCreatedEventTestFactory.create(command));

			verify(forwardingService).markAsFailed(command);
		}

	}
}