/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import static org.mockito.ArgumentMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.core.task.TaskExecutor;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.security.test.context.support.WithMockUser;

import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.nachrichten.email.MailSentEventTestFactory;
import de.ozgcloud.nachrichten.email.MailService;
import de.ozgcloud.vorgang.callcontext.CallContextTestFactory;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;

@DataITCase
public class ForwardingITCase {

	@Autowired
	private ApplicationEventPublisher eventPublisher;
	@Autowired
	private MongoOperations mongoOperations;
	@Autowired
	private TaskExecutor taskExecutor;
	@MockBean
	private MailService mailService;

	@Nested
	class TestAcknowledgeForwarding {

		private Vorgang vorgang;

		@BeforeEach
		void setup() {
			vorgang = VorgangTestFactory.create();
			mongoOperations.save(vorgang);
		}

		@Test
		@WithMockUser(username = CallContextTestFactory.CLIENT)
		void shouldCallMailService() {
			var forwarding = ForwardingTestFactory.createBuilder().vorgangId(vorgang.getId()).build();

			publishEventInNewThread(MailSentEventTestFactory.create(forwarding));

			Mockito.verify(mailService, Mockito.timeout(1000)).sendMail(any());
		}

		private void publishEventInNewThread(ApplicationEvent event) {
			taskExecutor.execute(() -> eventPublisher.publishEvent(event));
		}
	}
}
